/****************************************************************************
*  Copyright (c) 2011-2014 by Michael Fischer.
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  29.11.2011  mifi  First Version, tested with an Altera DE1 board.
*                    Used for the ChibiOS tutorial here:
*                    http://www.emb4fun.de/howto.html
*  01.05.2014  mifi  Tested with ChibiOS v2.4.6.
****************************************************************************/
#define __MAIN_C__

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include "ch.h"
#include "altera_avalon_pio_regs.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/

/*
 * Priority and stack size of the task.
 */
#define TASK_SEG7_PRIORITY (NORMALPRIO + 4)
#define TASK_HP_PRIORITY   (NORMALPRIO + 3)
#define TASK_LP_PRIORITY   (NORMALPRIO + 2)

#define TASK_SEG7_STK_SIZE 64
#define TASK_HP_STK_SIZE   64
#define TASK_LP_STK_SIZE   64

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/

static WORKING_AREA(SEG7Stack, TASK_SEG7_STK_SIZE);
static WORKING_AREA(HPStack,   TASK_HP_STK_SIZE);
static WORKING_AREA(LPStack,   TASK_LP_STK_SIZE);

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/

/***************************************************************************/
/*  HardwareInit                                                           */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void HardwareInit (void)
{
   /* Switch off 7-segment display */
   IOWR_ALTERA_AVALON_PIO_DATA(PIO_7SEG_BASE, 0xFFFFFFFF);
   
   /* Switch of all green LEDs */   
   IOWR_ALTERA_AVALON_PIO_DATA(PIO_LED_GREEN_BASE, 0x0000);

   /* Switch of all red LEDs */   
   IOWR_ALTERA_AVALON_PIO_DATA(PIO_LED_RED_BASE, 0x0000);

} /* HardwareInit */

/***************************************************************************/
/*  GetSegData                                                             */
/*                                                                         */
/*  Convert the given nibble into a value which can be used                */
/*  for the 7-segment display.                                             */
/*                                                                         */
/*  In    : Nibble to decode                                               */
/*  Out   : none                                                           */
/*  Return: Segment data                                                   */
/***************************************************************************/
static uint32_t GetSegData (uint8_t Nibble)
{
   uint32_t Value = 0;
   
   switch (Nibble)
   {
      case 0x00: Value = 0x40; break;
      case 0x01: Value = 0x79; break;
      case 0x02: Value = 0x24; break;
      case 0x03: Value = 0x30; break;
      case 0x04: Value = 0x19; break;
      case 0x05: Value = 0x12; break;
      case 0x06: Value = 0x02; break;
      case 0x07: Value = 0x78; break;
      case 0x08: Value = 0x00; break;
      case 0x09: Value = 0x18; break;
      case 0x0A: Value = 0x08; break;
      case 0x0B: Value = 0x03; break;
      case 0x0C: Value = 0x46; break;
      case 0x0D: Value = 0x21; break;
      case 0x0E: Value = 0x06; break;
      case 0x0F: Value = 0x0E; break;
   }
   
   return(Value);
} /* GetSegData */

/***************************************************************************/
/*  SEG7Task                                                               */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static msg_t SEG7Task (void *p)
{
   uint16_t Counter = 0;
   uint32_t Seg7Value;
   uint32_t Hex0, Hex1, Hex2, Hex3;
   
   p = p;
   
   while(1)
   {      
      Seg7Value = 0;
      
      Hex0 = GetSegData((Counter & 0x000F) >>  0);
      Hex1 = GetSegData((Counter & 0x00F0) >>  4);
      Hex2 = GetSegData((Counter & 0x0F00) >>  8);
      Hex3 = GetSegData((Counter & 0xF000) >> 12);
   
      Seg7Value = (Hex3 << 21) | (Hex2 << 14) | (Hex1 << 7) | Hex0;
      IOWR_ALTERA_AVALON_PIO_DATA(PIO_7SEG_BASE, Seg7Value);
      Counter++;
   
      chThdSleepMilliseconds(50);
   }
   
   /*
    * This return here make no sense.
    * But prevent a compiler warning.
    */
   return(0);    
} /* HPTask */

/***************************************************************************/
/*  HPTask                                                                 */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static msg_t HPTask (void *p)
{
   uint16_t Counter = 0;
   uint16_t Value   = 0x0001;
   
   p = p;
   
   while(1)
   {
      IOWR_ALTERA_AVALON_PIO_DATA(PIO_LED_GREEN_BASE, Value);
      
      Value = Value << 1;
      Counter++;
      if (8 == Counter)
      {
         Counter = 0;
         Value   = 1;
      }
      chThdSleepMilliseconds(100);
   }
   
   /*
    * This return here make no sense.
    * But prevent a compiler warning.
    */
   return(0);    
} /* HPTask */

/***************************************************************************/
/*  LPTask                                                                 */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static msg_t LPTask (void *p)
{
   uint16_t Counter = 0;
   uint16_t Value   = 0x0100;

   p = p;
   
   while(1)
   {
      IOWR_ALTERA_AVALON_PIO_DATA(PIO_LED_RED_BASE, Value);
      
      Value = Value >> 1;
      Counter++;
      if (9 == Counter)
      {
         Counter = 0;
         Value   = 0x0100;
      }
      chThdSleepMilliseconds(100);
   }
   
   /*
    * This return here make no sense.
    * But prevent a compiler warning.
    */
   return(0);
} /* LPTask */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/
/***************************************************************************/
/*  main                                                                   */
/*                                                                         */
/*  Note: ChibiOS/RT initialization.                                       */
/*        The ChibiOS/RT system was still init (chSysInit) before          */
/*        by the Nios system (ALT_OS_INIT). At this point all ChibiOS/RT   */
/*        function can be used.                                            */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
int main (void)
{
   /*
    * Before the application which is using ChibiOS/RT can be started,
    * the hardware of the target must be initialized.
    */
   HardwareInit();

   /*
    * Create our threads
    */
   chThdCreateStatic(SEG7Stack, sizeof(SEG7Stack), TASK_SEG7_PRIORITY, SEG7Task, NULL);
   chThdCreateStatic(HPStack, sizeof(HPStack), TASK_HP_PRIORITY, HPTask, NULL);
   chThdCreateStatic(LPStack, sizeof(LPStack), TASK_LP_PRIORITY, LPTask, NULL);
   
   /*
    * Normal main() thread activity.
    */
   while(1) 
   {
      chThdSleepMilliseconds(500);
   }
   
   /*
    * This return here make no sense. But to prevent the compiler warning:
    * "return type of 'main' is not 'int'
    * An int as return is used :-)
    */ 
   return(0);
} /* main */

/*** EOF ***/
